@@ -53,16 +53,14 @@
 #'
 #'   `isSQLKeyword` returns a logical vector parallel to `name`.
 #' @export
-setGeneric(
-  "make.db.names",
+setGeneric("make.db.names",
+  def = function(dbObj, snames, keywords = .SQL92Keywords, unique = TRUE, allow.keywords = TRUE, ...) standardGeneric("make.db.names"),
   signature = c("dbObj", "snames"),
-  function(dbObj, snames, keywords = .SQL92Keywords, unique = TRUE, allow.keywords = TRUE, ...)
-    standardGeneric("make.db.names"),
   valueClass = "character"
 )
 
 #' @rdname hidden_aliases
-setMethod("make.db.names", signature(dbObj="DBIObject", snames="character"),
+setMethod("make.db.names", signature(dbObj = "DBIObject", snames = "character"),
   definition = function(dbObj, snames, keywords, unique, allow.keywords, ...) {
     make.db.names.default(snames, keywords, unique, allow.keywords)
   },
@@ -77,41 +75,41 @@ setMethod("make.db.names", signature(dbObj="DBIObject", snames="character"),
 make.db.names.default <- function(snames, keywords = .SQL92Keywords,
                                   unique = TRUE, allow.keywords = TRUE) {
   makeUnique <- function(x, sep = "_") {
-    if(length(x)==0) return(x)
+    if (length(x) == 0) return(x)
     out <- x
-    lc <- make.names(tolower(x), unique=FALSE)
+    lc <- make.names(tolower(x), unique = FALSE)
     i <- duplicated(lc)
     lc <- make.names(lc, unique = TRUE)
-    out[i] <- paste(out[i], substring(lc[i], first=nchar(out[i])+1), sep=sep)
+    out[i] <- paste(out[i], substring(lc[i], first = nchar(out[i]) + 1), sep = sep)
     out
   }
   ## Note: SQL identifiers *can* be enclosed in double or single quotes
   ## when they are equal to reserverd keywords.
   fc <- substring(snames, 1, 1)
   lc <- substring(snames, nchar(snames))
-  i <- match(fc, c("'", '"'), 0)>0 & match(lc, c("'", '"'), 0)>0
-  snames[!i]  <- make.names(snames[!i], unique=FALSE)
-  if(unique)
+  i <- match(fc, c("'", '"'), 0) > 0 & match(lc, c("'", '"'), 0) > 0
+  snames[!i] <- make.names(snames[!i], unique = FALSE)
+  if (unique) {
     snames[!i] <- makeUnique(snames[!i])
-  if(!allow.keywords){
+  }
+  if (!allow.keywords) {
     kwi <- match(keywords, toupper(snames), nomatch = 0L)
-    snames[kwi] <- paste('"', snames[kwi], '"', sep='')
+    snames[kwi] <- paste('"', snames[kwi], '"', sep = "")
   }
   gsub("\\.", "_", snames)
 }
 
 #' @rdname make.db.names
 #' @export
-setGeneric(
-  "isSQLKeyword",
-  signature = c("dbObj", "name"),
-  function(dbObj, name, keywords = .SQL92Keywords, case = c("lower", "upper", "any")[3], ...)
+setGeneric("isSQLKeyword",
+  def = function(dbObj, name, keywords = .SQL92Keywords, case = c("lower", "upper", "any")[3], ...)
     standardGeneric("isSQLKeyword"),
+  signature = c("dbObj", "name"),
   valueClass = "logical"
 )
 
 #' @rdname hidden_aliases
-setMethod("isSQLKeyword", signature(dbObj="DBIObject", name="character"),
+setMethod("isSQLKeyword", signature(dbObj = "DBIObject", name = "character"),
   definition = function(dbObj, name, keywords, case, ...)
     isSQLKeyword.default(name, keywords, case),
   valueClass = "logical"
@@ -121,42 +119,45 @@ setMethod("isSQLKeyword", signature(dbObj="DBIObject", name="character"),
 #' @export
 isSQLKeyword.default <- function(name, keywords = .SQL92Keywords,
                                  case = c("lower", "upper", "any")[3]) {
-  n <- pmatch(case, c("lower", "upper", "any"), nomatch=0)
-  if(n==0)
+  n <- pmatch(case, c("lower", "upper", "any"), nomatch = 0)
+  if (n == 0) {
     stop('case must be one of "lower", "upper", or "any"')
+  }
   kw <- switch(c("lower", "upper", "any")[n],
     lower = tolower(keywords),
     upper = toupper(keywords),
-    any = toupper(keywords))
-  if(n==3)
+    any = toupper(keywords)
+  )
+  if (n == 3) {
     name <- toupper(name)
-  match(name, keywords, nomatch=0) > 0
+  }
+  match(name, keywords, nomatch = 0) > 0
 }
 
 ## SQL ANSI 92 (plus ISO's) keywords --- all 220 of them!
 ## (See pp. 22 and 23 in X/Open SQL and RDA, 1994, isbn 1-872630-68-8)
 
 #' @export
-setGeneric(
-  "SQLKeywords",
-  function(dbObj, ...) standardGeneric("SQLKeywords"),
+setGeneric("SQLKeywords",
+  def = function(dbObj, ...) standardGeneric("SQLKeywords"),
   valueClass = "character"
 )
 
 #' @rdname hidden_aliases
-setMethod("SQLKeywords", "DBIObject",
+setMethod("SQLKeywords", signature("DBIObject"),
   definition = function(dbObj, ...) .SQL92Keywords,
   valueClass = "character"
 )
 
 #' @rdname hidden_aliases
-setMethod("SQLKeywords", "missing",
+setMethod("SQLKeywords", signature("missing"),
   definition = function(dbObj, ...) .SQL92Keywords,
   valueClass = "character"
 )
 
 #' @export
-.SQL92Keywords <- c("ABSOLUTE", "ADD", "ALL", "ALLOCATE", "ALTER", "AND", "ANY",
+.SQL92Keywords <- c(
+  "ABSOLUTE", "ADD", "ALL", "ALLOCATE", "ALTER", "AND", "ANY",
   "ARE", "AS", "ASC", "ASSERTION", "AT", "AUTHORIZATION", "AVG", "BEGIN",
   "BETWEEN", "BIT", "BIT_LENGTH", "BY", "CASCADE", "CASCADED", "CASE", "CAST",
   "CATALOG", "CHAR", "CHARACTER", "CHARACTER_LENGTH", "CHAR_LENGTH",
@@ -200,8 +201,7 @@ setMethod("SQLKeywords", "missing",
 #' @inheritParams dbGetQuery
 #' @keywords internal
 #' @export
-setGeneric(
-  "dbCallProc",
+setGeneric("dbCallProc",
   def = function(conn, ...) {
     .Deprecated()
     standardGeneric("dbCallProc")
@@ -232,8 +232,7 @@ dbGetDBIVersion <- function() {
 #' @keywords internal
 #' @param flds a field description object as returned by `dbColumnInfo`.
 #' @export
-setGeneric(
-  "dbSetDataMappings",
+setGeneric("dbSetDataMappings",
   def = function(res, flds, ...) {
     .Deprecated()
     standardGeneric("dbSetDataMappings")
