@@ -22,25 +22,40 @@
 #'
 #' # This is safe because the single quote has been double escaped
 #' sqlInterpolate(ANSI(), sql, name = "H'); DROP TABLE--;")
-setGeneric(
-  "sqlInterpolate",
-  function(conn, sql, ..., .dots = list()) standardGeneric("sqlInterpolate")
+setGeneric("sqlInterpolate",
+  def = function(conn, sql, ..., .dots = list()) standardGeneric("sqlInterpolate")
 )
 
 #' @rdname hidden_aliases
 #' @export
-setMethod("sqlInterpolate", "DBIConnection", function(conn, sql, ..., .dots = list()) {
+setMethod("sqlInterpolate", signature("DBIConnection"), function(conn, sql, ..., .dots = list()) {
   pos <- sqlParseVariables(conn, sql)
 
-  if (length(pos$start) == 0)
+  if (length(pos$start) == 0) {
     return(SQL(sql))
+  }
 
   vars <- substring(sql, pos$start + 1, pos$end)
+  positional_vars <- pos$start == pos$end
+  if (all(positional_vars) != any(positional_vars)) {
+    stop("Can't mix positional (?) and named (?asdf) variables", call. = FALSE)
+  }
+
   values <- c(list(...), .dots)
-  if (!setequal(vars, names(values))) {
-    stop("Supplied vars don't match vars to interpolate", call. = FALSE)
+  if (all(positional_vars)) {
+    if (length(vars) != length(values)) {
+      stop("Supplied values don't match positional vars to interpolate", call. = FALSE)
+    }
+    if (any(names(values) != "")) {
+      stop("Positional variables don't take named arguments")
+    }
+  }
+  else {
+    if (!setequal(vars, names(values))) {
+      stop("Supplied values don't match named vars to interpolate", call. = FALSE)
+    }
+    values <- values[vars]
   }
-  values <- values[vars]
 
   safe_values <- vapply(values, function(x) {
     if (is.character(x)) {
@@ -89,15 +104,15 @@ setMethod("sqlInterpolate", "DBIConnection", function(conn, sql, ..., .dots = li
 #'   list(sqlQuoteSpec("'", "'"), sqlQuoteSpec('"', '"')),
 #'   list(sqlCommentSpec("#", "\n", FALSE))
 #' )
-setGeneric(
-  "sqlParseVariables",
-  function(conn, sql, ...) standardGeneric("sqlParseVariables")
+setGeneric("sqlParseVariables",
+  def = function(conn, sql, ...) standardGeneric("sqlParseVariables")
 )
 
 #' @rdname hidden_aliases
 #' @export
-setMethod("sqlParseVariables", "DBIConnection", function(conn, sql, ...) {
-  sqlParseVariablesImpl(sql,
+setMethod("sqlParseVariables", signature("DBIConnection"), function(conn, sql, ...) {
+  sqlParseVariablesImpl(
+    sql,
     list(
       sqlQuoteSpec('"', '"'),
       sqlQuoteSpec("'", "'")
@@ -112,13 +127,13 @@ setMethod("sqlParseVariables", "DBIConnection", function(conn, sql, ...) {
 #' @export
 #' @rdname sqlParseVariables
 sqlCommentSpec <- function(start, end, endRequired) {
-  list(start, end, endRequired)
+  list(start = start, end = end, endRequired = endRequired)
 }
 
 #' @export
 #' @rdname sqlParseVariables
 sqlQuoteSpec <- function(start, end, escape = "", doubleEscape = TRUE) {
-  list(start, end, escape, doubleEscape)
+  list(start = start, end = end, escape = escape, doubleEscape = doubleEscape)
 }
 
 #' @export
@@ -129,7 +144,9 @@ sqlQuoteSpec <- function(start, end, escape = "", doubleEscape = TRUE) {
 #' @param comments A list of `CommentSpec` calls defining the commenting
 #'   specification.
 sqlParseVariablesImpl <- function(sql, quotes, comments) {
-  sql_arr <- c(strsplit(as.character(sql), "", fixed = TRUE)[[1]], " ")
+  str_to_vec <- function(s) strsplit(s, "", fixed = TRUE)[[1L]]
+
+  sql_arr <- c(str_to_vec(as.character(sql)), " ")
 
   # characters valid in variable names
   var_chars <- c(LETTERS, letters, 0:9, "_")
@@ -144,45 +161,45 @@ sqlParseVariablesImpl <- function(sql, quotes, comments) {
   sql_variable_start <- 0L
 
   # prepare comments & quotes for quicker comparisions
-  for(c in seq_along(comments)) {
-    comments[[c]][[1]] <- strsplit(comments[[c]][[1]], "", fixed = TRUE)[[1]]
-    comments[[c]][[2]] <- strsplit(comments[[c]][[2]], "", fixed = TRUE)[[1]]
+  for (c in seq_along(comments)) {
+    comments[[c]][["start"]] <- str_to_vec(comments[[c]][["start"]])
+    comments[[c]][["end"]]   <- str_to_vec(comments[[c]][["end"]])
   }
-  for(q in seq_along(quotes)) {
-    quotes[[q]][[5]] <- nchar(quotes[[q]][[3]]) > 0L
+  for (q in seq_along(quotes)) {
+    quotes[[q]][["hasEscape"]] <- nchar(quotes[[q]][["escape"]]) > 0L
   }
 
-  state <- 'default'
-  i <- 0
-  while(i < length(sql_arr)) {
-    i <- i + 1
+  state <- "default"
+  i <- 0L
+  while (i < length(sql_arr)) {
+    i <- i + 1L
     switch(state,
 
       default = {
-        # variable?
+        #  variable
         if (sql_arr[[i]] == "?") {
           sql_variable_start <- i
-          state <- 'variable'
+          state <- "variable"
           next
         }
-        # starting quoted area?
-        for(q in seq_along(quotes)) {
-          if (identical(sql_arr[[i]], quotes[[q]][[1]])) {
+        # starting quoted area
+        for (q in seq_along(quotes)) {
+          if (identical(sql_arr[[i]], quotes[[q]][["start"]])) {
             quote_spec_offset <- q
-            state <- 'quote'
+            state <- "quote"
             break
           }
         }
         # we can abort here if the state has changed
-        if (state != 'default') next
-        # starting comment?
-        for(c in seq_along(comments)) {
-          comment_start_arr <- comments[[c]][[1]]
+        if (state != "default") next
+        # starting comment
+        for (c in seq_along(comments)) {
+          comment_start_arr <- comments[[c]][["start"]]
           comment_start_length <- length(comment_start_arr)
-          if (identical(sql_arr[i:(i + comment_start_length - 1)], comment_start_arr)) {
+          if (identical(sql_arr[i:(i + comment_start_length - 1L)], comment_start_arr)) {
             comment_spec_offset <- c
             i <- i + comment_start_length
-            state <- 'comment'
+            state <- "comment"
             break
           }
         }
@@ -190,40 +207,36 @@ sqlParseVariablesImpl <- function(sql, quotes, comments) {
 
       variable = {
         if (!(sql_arr[[i]] %in% var_chars)) {
-          # make sure variable has at least one character after the '?'
-          if (i - sql_variable_start < 2) {
-            stop("Length 0 variable")
-          }
           # append current variable offsets to return vectors
           var_pos_start <- c(var_pos_start, sql_variable_start)
-          var_pos_end <- c(var_pos_end, i - 1)
           # we have already read too much, go back
-          i <- i - 1
-          state <- 'default'
+          i <- i - 1L
+          var_pos_end <- c(var_pos_end, i)
+          state <- "default"
         }
       },
 
       quote = {
         # if we see backslash-like escapes, ignore next character
-        if (quotes[[quote_spec_offset]][[5]] && identical(sql_arr[[i]], quotes[[quote_spec_offset]][[3]])) {
-          i <- i + 1
+        if (quotes[[quote_spec_offset]][["hasEscape"]] && identical(sql_arr[[i]], quotes[[quote_spec_offset]][[3]])) {
+          i <- i + 1L
           next
         }
-        # end quoted area?
-        if (identical(sql_arr[[i]], quotes[[quote_spec_offset]][[2]])) {
+        # end quoted area
+        if (identical(sql_arr[[i]], quotes[[quote_spec_offset]][["end"]])) {
           quote_spec_offset <- 0L
-          state <- 'default'
+          state <- "default"
         }
       },
 
       comment = {
-        # end commented area?
-        comment_end_arr <- comments[[comment_spec_offset]][[2]]
+        # end commented area
+        comment_end_arr <- comments[[comment_spec_offset]][["end"]]
         comment_end_length <- length(comment_end_arr)
-        if (identical(sql_arr[i:(i + comment_end_length - 1)], comment_end_arr)) {
+        if (identical(sql_arr[i:(i + comment_end_length - 1L)], comment_end_arr)) {
           i <- i + comment_end_length
           comment_spec_offset <- 0L
-          state <- 'default'
+          state <- "default"
         }
       }
     ) # </switch>
@@ -232,8 +245,8 @@ sqlParseVariablesImpl <- function(sql, quotes, comments) {
   if (quote_spec_offset > 0L) {
     stop("Unterminated literal")
   }
-  if (comment_spec_offset > 0L && comments[[comment_spec_offset]][[3]]) {
+  if (comment_spec_offset > 0L && comments[[comment_spec_offset]][["endRequired"]]) {
     stop("Unterminated comment")
   }
-  list(start = as.integer(var_pos_start), end = as.integer(var_pos_end))
+  list(start = var_pos_start, end = var_pos_end)
 }
